/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * RobotManagerPanel.java
 *
 * Created on Dec 7, 2011, 5:24:08 PM
 */
package org.rwshop.swing.motion.robot.manager;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import org.robokind.api.motion.Robot;
import org.robokind.api.motion.utils.RobotController;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class RobotControllerPanel extends javax.swing.JPanel implements PropertyChangeListener {
    private RobotController myRobotController;
    
    /** Creates new form RobotManagerPanel */
    public RobotControllerPanel() {
        initComponents();
        repopulatePanel();
    }
    
    public void setRobotController(RobotController robotController){
        if(myRobotController != null){
            myRobotController.removePropertyChangeListener(this);
        }
        myRobotController = robotController;
        if(myRobotController != null){
            myRobotController.addPropertyChangeListener(this);
        }
        repopulatePanel();
    }
    
    private Robot getRobot(){
        if(myRobotController == null){
            return null;
        }
        return myRobotController.getRobot();
    }
    
    private void repopulatePanel(){
        populateInfo();
        populateConnection();
        populateEnabled();
        populateSelected();
        populateDefPos();
    }
    
    private void populateInfo(){
        Robot r = getRobot();
        String id = r == null ? "--" : r.getRobotId().getRobtIdString();
        lblRobotId.setText(id);
        String type = r == null ? "--" : r.getClass().getSimpleName();
        lblRobotType.setText(type);
    }
    
    private void populateConnection(){
        Robot r = getRobot();
        if(r == null){
            lblConnection.setText("--");
            btnConnect.setEnabled(false);
            btnDisconnect.setEnabled(false);
        }else if(r.isConnected()){
            lblConnection.setText("Connected");
            btnConnect.setEnabled(false);
            btnDisconnect.setEnabled(true);
        }else{
            lblConnection.setText("Disconnected");
            btnConnect.setEnabled(true);
            btnDisconnect.setEnabled(false);
        }
        populateEnabled();
    }
    
    private void populateEnabled(){
        Robot r = getRobot();
        if(r == null){
            lblEnabled.setText("--");
            btnEnable.setEnabled(false);
            btnDisable.setEnabled(false);
        }else if(!r.isConnected()){
            lblEnabled.setText("Disabled");
            btnEnable.setEnabled(false);
            btnDisable.setEnabled(false);
        }else if(r.isEnabled()){
            lblEnabled.setText("Enabled");
            btnEnable.setEnabled(false);
            btnDisable.setEnabled(true);
        }else{
            lblEnabled.setText("Disabled");
            btnEnable.setEnabled(true);
            btnDisable.setEnabled(false);
        }
        populateDefPos();
    }
    
    private void populateSelected(){
        //if(getRobot() == null){
            lblSelected.setText("--");
            btnSelect.setEnabled(false);
        //}else{
        //    lblSelected.setText("Select");
        //    btnSelect.setEnabled(true);
        //}
    }
    
    private void populateDefPos(){
        Robot r = getRobot();
        if(r == null){
            btnSetDefaultPos.setEnabled(false);
        }else if(!r.isConnected() || !r.isEnabled()){
            btnSetDefaultPos.setEnabled(false);
        }else{
            btnSetDefaultPos.setEnabled(true);
        }
    }
    

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        pnlRobotInfo = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        lblRobotId = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        lblRobotType = new javax.swing.JLabel();
        pnlConnection = new javax.swing.JPanel();
        lblConnection = new javax.swing.JLabel();
        btnConnect = new javax.swing.JButton();
        btnDisconnect = new javax.swing.JButton();
        pnlEnabled = new javax.swing.JPanel();
        lblEnabled = new javax.swing.JLabel();
        btnEnable = new javax.swing.JButton();
        btnDisable = new javax.swing.JButton();
        btnSetDefaultPos = new javax.swing.JButton();
        pnlSelected = new javax.swing.JPanel();
        lblSelected = new javax.swing.JLabel();
        btnSelect = new javax.swing.JButton();
        btnRemove = new javax.swing.JButton();

        jLabel1.setText("Robot Id:");

        lblRobotId.setText("robot.id");

        jLabel2.setText("Type:");

        lblRobotType.setText("Robot");

        javax.swing.GroupLayout pnlRobotInfoLayout = new javax.swing.GroupLayout(pnlRobotInfo);
        pnlRobotInfo.setLayout(pnlRobotInfoLayout);
        pnlRobotInfoLayout.setHorizontalGroup(
            pnlRobotInfoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pnlRobotInfoLayout.createSequentialGroup()
                .addGroup(pnlRobotInfoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                    .addComponent(jLabel2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jLabel1, javax.swing.GroupLayout.Alignment.LEADING))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(pnlRobotInfoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(lblRobotId)
                    .addComponent(lblRobotType)))
        );
        pnlRobotInfoLayout.setVerticalGroup(
            pnlRobotInfoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pnlRobotInfoLayout.createSequentialGroup()
                .addGroup(pnlRobotInfoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(lblRobotId))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(pnlRobotInfoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(lblRobotType)))
        );

        lblConnection.setText("Status");
        lblConnection.setMaximumSize(new java.awt.Dimension(64, 14));
        lblConnection.setMinimumSize(new java.awt.Dimension(64, 14));
        lblConnection.setPreferredSize(new java.awt.Dimension(64, 14));

        btnConnect.setText("Connect");
        btnConnect.setMaximumSize(new java.awt.Dimension(85, 23));
        btnConnect.setMinimumSize(new java.awt.Dimension(85, 23));
        btnConnect.setPreferredSize(new java.awt.Dimension(85, 23));
        btnConnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnConnectActionPerformed(evt);
            }
        });

        btnDisconnect.setText("Disconnect");
        btnDisconnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDisconnectActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout pnlConnectionLayout = new javax.swing.GroupLayout(pnlConnection);
        pnlConnection.setLayout(pnlConnectionLayout);
        pnlConnectionLayout.setHorizontalGroup(
            pnlConnectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pnlConnectionLayout.createSequentialGroup()
                .addComponent(lblConnection, javax.swing.GroupLayout.PREFERRED_SIZE, 64, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(btnConnect, javax.swing.GroupLayout.PREFERRED_SIZE, 85, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnDisconnect, javax.swing.GroupLayout.PREFERRED_SIZE, 85, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        pnlConnectionLayout.setVerticalGroup(
            pnlConnectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pnlConnectionLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(lblConnection, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addComponent(btnDisconnect, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addComponent(btnConnect, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        lblEnabled.setText("Status");
        lblEnabled.setMaximumSize(new java.awt.Dimension(64, 14));
        lblEnabled.setMinimumSize(new java.awt.Dimension(64, 14));
        lblEnabled.setPreferredSize(new java.awt.Dimension(64, 14));

        btnEnable.setText("Enable");
        btnEnable.setMaximumSize(new java.awt.Dimension(85, 23));
        btnEnable.setMinimumSize(new java.awt.Dimension(85, 23));
        btnEnable.setPreferredSize(new java.awt.Dimension(85, 23));
        btnEnable.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnEnableActionPerformed(evt);
            }
        });

        btnDisable.setText("Disable");
        btnDisable.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnDisableActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout pnlEnabledLayout = new javax.swing.GroupLayout(pnlEnabled);
        pnlEnabled.setLayout(pnlEnabledLayout);
        pnlEnabledLayout.setHorizontalGroup(
            pnlEnabledLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pnlEnabledLayout.createSequentialGroup()
                .addComponent(lblEnabled, javax.swing.GroupLayout.PREFERRED_SIZE, 64, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(btnEnable, javax.swing.GroupLayout.PREFERRED_SIZE, 85, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnDisable, javax.swing.GroupLayout.PREFERRED_SIZE, 85, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        pnlEnabledLayout.setVerticalGroup(
            pnlEnabledLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pnlEnabledLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(lblEnabled, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addComponent(btnDisable, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addComponent(btnEnable, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        btnSetDefaultPos.setText("Set Default Positions");
        btnSetDefaultPos.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSetDefaultPosActionPerformed(evt);
            }
        });

        lblSelected.setText("Status");
        lblSelected.setMaximumSize(new java.awt.Dimension(64, 14));
        lblSelected.setMinimumSize(new java.awt.Dimension(64, 14));
        lblSelected.setPreferredSize(new java.awt.Dimension(64, 14));

        btnSelect.setText("Set Selected");
        btnSelect.setMaximumSize(new java.awt.Dimension(85, 23));
        btnSelect.setMinimumSize(new java.awt.Dimension(85, 23));
        btnSelect.setPreferredSize(new java.awt.Dimension(85, 23));
        btnSelect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnSelectActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout pnlSelectedLayout = new javax.swing.GroupLayout(pnlSelected);
        pnlSelected.setLayout(pnlSelectedLayout);
        pnlSelectedLayout.setHorizontalGroup(
            pnlSelectedLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pnlSelectedLayout.createSequentialGroup()
                .addComponent(lblSelected, javax.swing.GroupLayout.PREFERRED_SIZE, 64, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(btnSelect, javax.swing.GroupLayout.DEFAULT_SIZE, 176, Short.MAX_VALUE))
        );
        pnlSelectedLayout.setVerticalGroup(
            pnlSelectedLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(pnlSelectedLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(lblSelected, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addComponent(btnSelect, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        btnRemove.setText("remove");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(pnlRobotInfo, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 78, Short.MAX_VALUE)
                        .addComponent(btnRemove))
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(btnSetDefaultPos, javax.swing.GroupLayout.PREFERRED_SIZE, 176, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                .addComponent(pnlSelected, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(pnlEnabled, javax.swing.GroupLayout.Alignment.LEADING, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(pnlConnection, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                        .addContainerGap())))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(pnlRobotInfo, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(btnRemove))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(pnlConnection, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(pnlEnabled, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(pnlSelected, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnSetDefaultPos, javax.swing.GroupLayout.PREFERRED_SIZE, 23, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void btnConnectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnConnectActionPerformed
        if(myRobotController == null){
            populateConnection();
            return;
        }
        myRobotController.connectRobot();
    }//GEN-LAST:event_btnConnectActionPerformed

    private void btnDisconnectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDisconnectActionPerformed
        if(myRobotController == null){
            populateConnection();
            return;
        }
        myRobotController.disconnectRobot();
    }//GEN-LAST:event_btnDisconnectActionPerformed

    private void btnEnableActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnEnableActionPerformed
        if(myRobotController == null){
            populateEnabled();
            return;
        }
        myRobotController.enableRobot();
    }//GEN-LAST:event_btnEnableActionPerformed

    private void btnDisableActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnDisableActionPerformed
        if(myRobotController == null){
            populateEnabled();
            return;
        }
        myRobotController.disableRobot();
    }//GEN-LAST:event_btnDisableActionPerformed

    private void btnSelectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSelectActionPerformed
        if(myRobotController == null){
            populateSelected();
            return;
        }
        myRobotController.selectRobot();
    }//GEN-LAST:event_btnSelectActionPerformed

    private void btnSetDefaultPosActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSetDefaultPosActionPerformed
        if(myRobotController == null){
            populateSelected();
            return;
        }
        myRobotController.setDefaultPositions();
    }//GEN-LAST:event_btnSetDefaultPosActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnConnect;
    private javax.swing.JButton btnDisable;
    private javax.swing.JButton btnDisconnect;
    private javax.swing.JButton btnEnable;
    private javax.swing.JButton btnRemove;
    private javax.swing.JButton btnSelect;
    private javax.swing.JButton btnSetDefaultPos;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel lblConnection;
    private javax.swing.JLabel lblEnabled;
    private javax.swing.JLabel lblRobotId;
    private javax.swing.JLabel lblRobotType;
    private javax.swing.JLabel lblSelected;
    private javax.swing.JPanel pnlConnection;
    private javax.swing.JPanel pnlEnabled;
    private javax.swing.JPanel pnlRobotInfo;
    private javax.swing.JPanel pnlSelected;
    // End of variables declaration//GEN-END:variables

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        if(evt == null){
            return;
        }else if(RobotController.PROP_ROBOT.equals(evt.getPropertyName())){
            repopulatePanel();
        //}else if(RobotController.PROP_SELECT_ROBOT.equals(evt.getPropertyName())){
        //    populateSelected();
        }else if(Robot.PROP_CONNECTED.equals(evt.getPropertyName())){
            populateConnection();
        }else if(Robot.PROP_ENABLED.equals(evt.getPropertyName())){
            populateEnabled();
        }
    }
}
